"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Auth = exports.Totp = void 0;
const rxjs_1 = require("rxjs");
const api_1 = require("./api");
const utils_1 = require("@ztimson/utils");
class Totp {
    api;
    constructor(api) {
        this.api = api;
    }
    disable(username) {
        return this.api.request({ url: `/api/auth/totp/${username}`, method: 'DELETE' });
    }
    enable = this.reset;
    reset(username) {
        return this.api.request({ url: `/api/auth/totp/${username}`, method: 'POST' });
    }
    setup(username, method = 'app', totp) {
        return this.api.request({ url: `/api/auth/totp/${username}`, body: (0, utils_1.clean)({
                method,
                totp
            }) });
    }
}
exports.Totp = Totp;
class Auth extends utils_1.TypedEmitter {
    opts;
    api;
    storageKey;
    totp;
    $user = new rxjs_1.BehaviorSubject(undefined);
    get user() { return this.$user.value; }
    set user(user) {
        if (!(0, utils_1.isEqual)(this.user, user)) {
            const u = user ? user : null;
            this.$user.next(u);
            this.emit('USER', u);
        }
    }
    constructor(api, opts) {
        super();
        this.opts = opts;
        this.api = typeof api == 'string' ? new api_1.Api(api) : api;
        this.totp = new Totp(this.api);
        if (!this.opts?.loginUi)
            this.opts = { ...this.opts, loginUi: this.api.url + '/ui/login' };
        this.storageKey = `momentum:${new URL(this.api.url).host}`;
        this.api.addInterceptor((resp, next) => {
            const blacklist = [
                '/api/auth/login',
                '/api/auth/password',
                '/api/auth/totp'
            ];
            if (resp.status == 401 && !blacklist.find(url => resp.url.includes(url)))
                this.emit('SESSION_EXPIRED');
            next();
        });
        this.api.on('TOKEN', token => {
            if (this.opts?.persist) {
                if (token)
                    localStorage.setItem(this.storageKey, token);
                else
                    localStorage.removeItem(this.storageKey);
            }
            if (token)
                this.session(token, true).catch(() => { });
            else
                this.user = null;
        });
        if (opts?.persist) {
            const token = localStorage.getItem(this.storageKey);
            if (token)
                this.api.token = token;
            else
                this.user = null;
        }
        else {
            this.user = null;
        }
    }
    knownHost(host = location.origin) {
        if (host.startsWith('/'))
            return Promise.resolve();
        return this.api.request({ url: `/api/auth/known-host?host=${encodeURI(new URL(host).origin)}` });
    }
    login(username, password, totp) {
        return this.api.request({
            url: '/api/auth/login',
            headers: { Authorization: undefined },
            body: {
                username: username.trim(),
                password: password.trim(),
                totp,
            }
        }).then(async (resp) => {
            this.api.token = resp.token;
            return await this.once('USER');
        });
    }
    loginRedirect(host = location.origin) {
        return new Promise((res, rej) => {
            const win = window.open(encodeURI(`${this.opts?.loginUi}?redirect=postmessage&host=${host}`), '_blank');
            if (!win)
                return rej('Unable to open login');
            win.addEventListener('message', (event) => {
                if (!event?.data?.token)
                    return rej('Unknown response from login');
                this.api.token = event.data.token;
                res(event.data.token);
                win.close();
            });
        });
    }
    logout() {
        this.api.token = null;
        this.user = null;
        this.emit('LOGOUT');
    }
    async register(u) {
        const user = await this.api.request({ url: '/api/auth/register', body: { ...u } });
        if (user.image.startsWith('/'))
            user.image = `${this.api.url}${user.image}?token=${this.api.token}`;
        this.emit('REGISTER', user);
        return user;
    }
    reset(emailOrPass, token) {
        return this.api.request({
            url: '/api/auth/reset',
            headers: { 'Authorization': token ? `Bearer ${token}` : undefined },
            body: {
                email: token ? undefined : emailOrPass,
                password: token ? emailOrPass : undefined,
            }
        });
    }
    async session(token, set = false) {
        if (!token)
            token = this.api.token;
        const user = await this.api.request({
            url: '/api/auth/session',
            headers: token ? { 'Authorization': `Bearer ${token}` } : undefined
        });
        if (set) {
            this.api.token = token;
            if (user?.user)
                user.user.image = `${this.api.url}${user.user.image}?token=${this.api.token}`;
            this.user = user?.user || null;
            if (user)
                this.emit('LOGIN', user.user);
        }
        return user;
    }
    async updatePassword(username, password, oldPassword) {
        return this.api.request({
            url: '/api/auth/password',
            body: { username, password, oldPassword }
        }).then(resp => {
            if (resp?.token)
                this.api.token = resp.token;
        });
    }
}
exports.Auth = Auth;
